//
// HtWordList.cc
//
// HtWordList: Specialized WordList class that can hold a list 
//	       of words waiting to be inserted in the database.
//
// Part of the ht://Dig package   <http://www.htdig.org/>
// Copyright (c) 1995-2002 The ht://Dig Group
// For copyright details, see the file COPYING in your distribution
// or the GNU Public License version 2 or later
// <http://www.gnu.org/copyleft/gpl.html>
//
// $Id: HtWordList.cc,v 1.4 2002/02/01 22:49:28 ghutchis Exp $
//

#ifdef HAVE_CONFIG_H
#include "htconfig.h"
#endif /* HAVE_CONFIG_H */

#include "HtWordList.h"
#include "HtWordReference.h"
#include "WordRecord.h"
#include "WordType.h"
#include "HtConfiguration.h"
#include "htString.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <iostream.h>
#include <fstream.h>
#include <errno.h>

//*****************************************************************************
// HtWordList::~HtWordList()
//
HtWordList::~HtWordList()
{
  if (_words)
    {
      delete _words;
      _words = NULL;
    }
  if (_wordlist)
    {
      if (_wordlist->isopen)
	_wordlist->Close();
      if(_wordlist->isopen) _wordlist->Close();
      delete _words;
      _words = NULL;
    }
  if (_config)
    {
      delete _config;
      _config = NULL;
    }
}

//*****************************************************************************
//
HtWordList::HtWordList(const HtConfiguration& config_arg) :
  WordContext(config_arg)
{
  _words = new HtList;
  _config = new HtConfiguration(config_arg);
  _wordlist = List();
}

//*****************************************************************************
//
void HtWordList::Replace(const HtWordReference& arg)
{
  //
  // New word.  Create a new reference for it and cache it in the object.
  //
  _words->Add((Object *) new HtWordReference(arg));
}

//*****************************************************************************
// void HtWordList::Flush()
//   Dump the current list of words to the database.  After
//   the words have been dumped, the list will be destroyed to make
//   room for the words of the next document.
//   
void HtWordList::Flush()
{
  HtWordReference	*wordRef;
  WordReference		*insertedRef;

  // Provided for backwards compatibility
  if (!_wordlist->isopen)
    _wordlist->Open(_config->Find("word_db"), O_RDWR);

  //  _wordlist->BatchStart();

  _words->Start_Get();
  while ((wordRef = (HtWordReference *) _words->Get_Next()))
    {
      if (wordRef->Word().length() == 0) {
	cerr << "HtWordList::Flush: unexpected empty word\n";
	continue;
      }
      
      // prepare a WordReference to insert
      //      insertedRef = Word(wordRef->Key(), wordRef->Record());
      //      cout << wordRef->Key() << endl;
      _wordlist->Override(_wordlist->Word(wordRef->Key()));
    }	
  //  _wordlist->BatchEnd();
    
  // Cleanup
  _words->Destroy();
}

//*****************************************************************************
// void HtWordList::Skip()
//   The current document has disappeared or been modified. 
//   We do not need to store these words.
//
void HtWordList::Skip()
{
  _words->Destroy();
}

//
// Callback data dedicated to Dump and dump_word communication
//
class DumpWordData : public Object
{
public:
  DumpWordData(FILE* fl_arg) { fl = fl_arg; }

  FILE* fl;
};

//*****************************************************************************
//
// Write the ascii representation of a word occurence. Helper
// of WordList::Dump
//
static int dump_word(WordList *, WordDBCursor &, const WordReference *word, Object &data)
{
  const HtWordReference *word_tmp = (const HtWordReference *)word;

  DumpWordData &info = (DumpWordData &)data;

  word_tmp->Dump(info.fl);

  return OK;
}

//*****************************************************************************
// int HtWordList::Dump(char* filename)
//
// Write an ascii version of the word database in <filename>
//
int HtWordList::Dump(const String& filename)
{
  FILE		*fl;

  if (!_wordlist->isopen) {
    _wordlist->Open(_config->Find("word_db"), O_RDONLY);
  }

  if((fl = fopen(filename, "w")) == 0) {
    perror(form("HtWordList::Dump: opening %s for writing", (const char*)filename));
    return NOTOK;
  }

  HtWordReference::DumpHeader(fl);
  DumpWordData data(fl);
  WordCursor* search = _wordlist->Cursor(dump_word, &data);
  search->Walk();
  delete search;
  
  fclose(fl);

  return OK;
}

//*****************************************************************************
// int HtWordList::Load(char* filename)
//
// Read in an ascii version of the word database in <filename>
//
int HtWordList::Load(const String& filename)
{
  FILE		*fl;
  String	data;
  HtWordReference *next;

  if (!_wordlist->isopen) {
    cerr << "HtWordList::Load: database must be opened first\n";
    return NOTOK;
  }

  if((fl = fopen(filename, "r")) == 0) {
    perror(form("HtWordList::Load: opening %s for reading", (const char*)filename));
    return NOTOK;
  }

  if (HtWordReference::LoadHeader(fl) != OK)
    {
      cerr << "HtWordList::Load: header is not correct\n";
      return NOTOK;
    }

  while (data.readLine(fl))
    {
      next = new HtWordReference;
      if (next->Load(data) != OK)
	{
	  delete next;
	  continue;
	}
  
      _words->Add((Object *)next);
    }

  Flush();
  fclose(fl);

  return OK;
}
