//
// qtest.cc
//
// qtest: A program to test the Query classes as replacement for the current
//            parsing code
//
// Part of the ht://Dig package   <http://www.htdig.org/>
// Copyright (c) 1999-2002 The ht://Dig Group
// For copyright details, see the file COPYING in your distribution
// or the GNU Public License version 2 or later
// <http://www.gnu.org/copyleft/gpl.html>
//
// $Id: qtest.cc,v 1.2 2002/02/02 18:18:13 ghutchis Exp $
//

#ifdef HAVE_CONFIG_H
#include "htconfig.h"
#endif /* HAVE_CONFIG_H */

#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include "cgi.h"
#include "defaults.h"
#include "WordContext.h"
#include <iostream.h>
#include "QueryParser.h"
#include "Query.h"
#include "ResultList.h"
#include "Exact.h"
#include "Accents.h"
#include "Prefix.h"
#include "WordSearcher.h"
#include "OrFuzzyExpander.h"
#include "ExactWordQuery.h"
#include "OrQueryParser.h"
#include "AndQueryParser.h"
#include "BooleanQueryParser.h"
#include "GParser.h"
#include "HtRegex.h"

// If we have this, we probably want it.
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

void reportError(char *msg);
void usage();

int			debug = 0;

// We want to make sure we timeout if a search takes way too long
//  (or someone could have an easy Denial of Service attack)
typedef void (*SIGNAL_HANDLER) (...);

void
ParseAndGet(QueryParser &parser, const String &string);

//*****************************************************************************
// int main()
//
int
main(int ac, char **av)
{
    int			c;
    extern char		*optarg;
    int		        override_config=0;
    String		configFile = DEFAULT_CONFIG_FILE;
    String		logicalWords;
    int			pageNumber = 1;
    HtRegex		limit_to;
    HtRegex		exclude_these;

     //
     // Parse command line arguments
     //
     while ((c = getopt(ac, av, "c:dv")) != -1)
     {
 	switch (c)
 	{
 	    case 'c':
              if (!getenv("REQUEST_METHOD"))
                {
                  configFile = optarg;
                  override_config=1;
                }
 	    case 'v':
 		debug++;
 		break;
 	    case 'd':
 		debug++;
 		break;
	    case '?':
	        usage();
                break;
 	}
     }

     // Set alarm() so the search will timeout (probably should be config)
     alarm(5*60);

    //
    // Parse the CGI parameters.
    //
    char	none[] = "";
    cgi		input(optind < ac ? av[optind] : none);

    String	 originalWords = input["words"];
    originalWords.chop(" \t\r\n");

    HtConfiguration* config= HtConfiguration::config();
    // Set up the config (probably should go in a separate procedure)
    config->Defaults(&defaults[0]);

    // To allow . in filename while still being 'secure',
    // e.g. htdig-f.q.d.n.conf
    if (!override_config && input.exists("config")
        && (strstr(input["config"], "./") == NULL))
      {
        char    *configDir = getenv("CONFIG_DIR");
        if (configDir)
	  {
            configFile = configDir;
	  }
        else
	  {
            configFile = CONFIG_DIR;
	  }
        if (strlen(input["config"]) == 0)
          configFile = DEFAULT_CONFIG_FILE;
        else
          configFile << '/' << input["config"] << ".conf";
      }

    if (access((char*)configFile, R_OK) < 0)
    {
	reportError(form("Unable to find configuration file '%s'",
			 configFile.get()));
    }
	
    config->Read(configFile);

    if (input.exists("method"))
      config->Add("match_method", input["method"]);
    if (input.exists("format"))
      config->Add("template_name", input["format"]);

    if (input.exists("matchesperpage"))
      {
        // minimum check for a valid int value of "matchesperpage" cgi variable
        if (atoi(input["matchesperpage"]) > 0)
	  config->Add("matches_per_page", input["matchesperpage"]);
      }

    if (input.exists("page"))
      pageNumber = atoi(input["page"]);
    if (input.exists("config"))
      config->Add("config", input["config"]);
    if (input.exists("restrict"))
      config->Add("restrict", input["restrict"]);
    if (input.exists("exclude"))
      config->Add("exclude", input["exclude"]);
    if (input.exists("sort"))
      config->Add("sort", input["sort"]);

    if (config->Find("restrict").length())
    {
	// Create a temporary list from either the configuration
	// file or the input parameter
	StringList l(config->Find("restrict"), " \t\r\n\001|");
	limit_to.setEscaped(l);
	String u = l.Join('|');
	config->Add("restrict", u);  // re-create the config attribute
    }
    if (config->Find("exclude").length())
    {
	// Create a temporary list from either the configuration
	// file or the input parameter
	StringList l(config->Find("exclude"), " \t\r\n\001|");
	exclude_these.setEscaped(l);
	String u = l.Join('|');
	config->Add("exclude", u);  // re-create the config attribute
    }

    // Initialize htword library (key description + wordtype...)
    WordContext::Initialize(*config);    

    // Set up fuzzy rankings from the search_algorithm
    StringList  algs(config->Find("search_algorithm"), " \t");
    List                algorithms;
    String              name, weight;
    double              fweight;
    Fuzzy               *fuzzy = 0;
    OrFuzzyExpander	exp;
    for (int i = 0; i < algs.Count(); i++)
      {
        name = strtok(algs[i], ":");
        weight = strtok(0, ":");
        if (name.length() == 0)
	  name = "exact";
        if (weight.length() == 0)
	  weight = "1";
        fweight = atof(weight);

        fuzzy = Fuzzy::getFuzzyByName(name.get(), config);
        if (fuzzy)
	  {
            fuzzy->setWeight(fweight);
            fuzzy->openIndex();
            exp.Add(fuzzy);
	  }
      }
    QueryParser::SetFuzzyExpander(&exp);

    WordSearcher searcher(config->Find("word_db"));
    ExactWordQuery::SetSearcher(&searcher);

    // -- put here your prefered cache
    //QueryCache *cache = new XXX;
    //Query::SetCache(cache);

    // This will eventually need to treat boolean_keywords, etc.
    if (config->Find("match_method") == "and" ||
	config->Find("match_method") == "all")
      {
	AndQueryParser parser;
	ParseAndGet(parser, originalWords);
      }
    else if (config->Find("match_method") == "or" ||
	config->Find("match_method") == "any")
      {
	OrQueryParser parser;
	ParseAndGet(parser, originalWords);    
      }
    else
      {
	BooleanQueryParser parser;
	ParseAndGet(parser, originalWords);    
      }

    if(doall || doand)
      {
	cout << "Trying and..." << endl;
	ParseAndGet(a, originalWords);
      }
}

void
ParseAndGet(QueryParser &parser, const String &query)
{
	Query *q = parser.Parse(query);
	if(q)
	{
		cout << "Parsed: " << q->GetLogicalWords() << endl;
		ResultList *l = q->GetResults();
		if(l)
		{
			cout << "Evaluated with " << l->Count() << " matches" << endl;
			if(debug) l->Dump();
		}
		else
		{
			cout << "No matches" << endl;;
		}
	}
	else
	{
		cerr << "syntax error: " << flush << parser.Error() << endl;
	}
	delete q;
}


//*****************************************************************************
// void usage()
//   Display program usage information--assumes we're running from a cmd line
//
void usage()
{
  cout << "usage: qtest [-a][-o][-b][-g][-v][-d][-c configfile]\n";
  cout << "This program is part of ht://Dig " << PACKAGE_VERSION << "\n\n";
  cout << "Options:\n";
  cout << "\t-v -d\tVerbose mode.  This increases the verbosity of the\n";
  cout << "\t\tprogram.  Using more than 2 is probably only useful\n";
  cout << "\t\tfor debugging purposes.  The default verbose mode\n";
  cout << "\t\tgives a progress on what it is doing and where it is.\n\n";
  cout << "\t-c configfile\n";
  cout << "\t\tUse the specified configuration file instead on the\n";
  cout << "\t\tdefault.\n\n";
  cout << "\t-a\tPerform only and/all parsing\n\n";
  cout << "\t-o\tPerform only or/any parsing\n\n";
  cout << "\t-b\tPerform only boolean parsing\n\n";
  cout << "\t-g\tPerform only no-precedence-boolean parsing\n\n";
  exit(0);
}

//*****************************************************************************
// Report an error and die
//
void reportError(char *msg)
{
    cout << "qtest: " << msg << "\n\n";
    exit(1);
}
